require 'cgi'

class FormattedText
  attr_accessor :text, :style_size, :color_options_til
  
  def initialize(text='', size=nil)
    @text = text.to_s
    @size = size
  end

  def plain_text # Remove images for the sake of space
    #FormattedText.replace_tags(@text, DELIMITERS, PATTERNS, PLAIN_REPLACE)
    text = html_text(false)
    text.gsub!(/<img[^>]*>/, '&lt;image&gt;')
    text
  end

  def html_text(apply_size = true)
    text = CGI::escapeHTML(@text)
#    text = FormattedText.replace_tags(text, DELIMITERS, PATTERNS, @HTML_REPLACE)
    text = FormattedText.replace_tags(text, @color_options_til)
    text = (apply_size and @style_size) ? "<span style='font-size:#{@style_size};'>\n" + text + "\n</span>" : text

    # Switch back the &'s changed to &amp; by escapeHTML
    text.gsub!(/&amp;/, '&')

    # Color all options of the form "A." till the last option letter specified
    # in @color_options_til.
    if color_options_til and ?A <= color_options_til and color_options_til <= ?Z
        text.gsub!(/((?:\s|(?:\&nbsp;)|>)+)([A-#{'%c' % color_options_til}]\.)/,
                  '\1<span class="choice">\2</span>')
    end

    text
  end

  def to_s
    @text
  end

  # Given an image directory, update the HTML image replacement tag.
  def FormattedText.image_dir=(dir)
    dir = dir.to_s
    dir = dir + '/' if dir.size > 0 and dir[-1] != ?/
    HTML_REPLACE[-1] = '<img src="' + dir + '\1" />'
  end

private

  # Replacements for formatting  
  HTML_TAGS  = [ 'b', 'i', 'u', 'sup', 'sub', 'pre', 'code', 'h1', 'h2', 'h3', 'em' ]
  COLOR_TAGS  = [ 'yellow', 'red', 'green', 'blue', 'white', 'gray', 'lime', 'cyan', 'magenta' ]
  CLASS_TAGS  = [ 'math', 'emph' ]
  DELIMITERS = [['\[', '\]'], ['\{', '\}'], ['\|', '\|'], ['\/', '\/']]
  PATTERNS   = [
        '-(' + HTML_TAGS.join('|') + ')\s+(.*?)-',  # Regular tags
        '-(' + COLOR_TAGS.join('|') + '|(?:#[0-9A-Fa-f]{6}))\s+(.*?)-',  # Color tags
        '-(' + CLASS_TAGS.join('|') + ')\s+(.*?)-',  # <span> class tags
        '-c\s+(.*?)-',
        '-img\s+(.*?)-'                        # Replace the image tag
    ]
  PLAIN_REPLACE = [
        '\2',
        '\2',
        '\2',
        '\1',
        '&lt;image&gt;'
    ]
  HTML_REPLACE = [
        '<\1>\2</\1>',
        '<span style="color:\1;">\2</span>',
        '<span class="\1">\2</span>',
        '<center>\1</center>',
        '<img src="../images/\1" />'
    ]
#  FormattedText.image_dir = '../images/'  # Sets the image replacement for html

  # Do an array of replacements
#  def FormattedText.replace_tags(text, delimiters, patterns, replacements)
  def FormattedText.replace_tags(text, color_options_til = nil)
    text = text.dup

    text.gsub!(/\n/, "<br />\n")

    for pair in DELIMITERS
      for i in 0...PATTERNS.size do
        pat = /#{pair[0]}#{PATTERNS[i]}#{pair[1]}/m
        text.gsub!(pat, HTML_REPLACE[i]);
      end
    end
    
    # Additionally, make all the <pre> bold so that they are more easily visible
    text.gsub!(/<pre>/, '<b><pre>')
    text.gsub!(/<\/pre>/, '</pre></b>')
    text.gsub!(/<code>/, '<b><code>')
    text.gsub!(/<\/code>/, '</code></b>')

    # Remove extraneous <br />'s from the content of <pre>'s
    text.gsub!(/<pre>(.*?)<\/pre>/im) { '<pre>' + $1.gsub!(/<br\s*\/?>$/mi, '') + '</pre>' }

    text
  end
end
